// GSL Optimization Example in MS Visual Studio .Net 2003 (Visual C++ 7)
//
// This example is the Nonlinear Least-Squares Fitting Example (expfit.c) found in the GSL documentation:  
// http://www.gnu.org/software/gsl/manual/html_node/Example-programs-for-Nonlinear-Least_002dSquares-Fitting.html
// 
// 2007/08/20 Miguel Torres Torriti
// --- Creating a project ---
//    Copy GSL include files to $MSVS/include/gsl
//    where $MSVS is the MS Visual C++ directory.
// 2. Copy GSL lib files to $MSVS/lib
//    where $MSVS is the MS Visual C++ directory.
// 3. Create a new project with the menu option:
//    File/New/Project/Visual C++ Projects/Win32/Win32 Console/Application Settings
//    -Select Console Application
//    -Select Empty Project
//4.a Add the file (gsl_optim_ex.h) to the project with the menu option
//    File/Add Existing Item or by right-clicking on "Header Files" within the Solution
//    Explorer pane (use CTRL+ALT+L to open the Solution Explorer) and then selecting 
//    Add/Add Existing Item...
//4.b Add the files (gsl_optim_ex.cpp, expfit.cpp) to the project with the menu option
//    File/Add Existing Item or by right-clicking on "Source Files" within the Solution
//    Explorer pane (use CTRL+ALT+L to open the Solution Explorer) and then selecting 
//    Add/Add Existing Item...
// 5. Open this file (gsl_optim_ex.cpp) within the editor window and place the cursor within
//    the editor window.
// 6. Select the menu option Project/gsl_optim_ex Properties.  Set the following options:
//    a. For Debug version:
//       Linker/General/Enable Incremental Linking = YES
//       Linker/Input/Additional Dependecies = gsl_d.lib
//       Linker/Input/Ignore Specific Library = libcd.lib
//    b. For Release version:
//       Linker/General/Enable Incremental Linking = NO
//       Linker/Input/Additional Dependecies = gsl.lib
//       Linker/Input/Ignore Specific Library = libc.lib
//

#include <gsl/gsl_vector.h>
#include <gsl/gsl_multifit_nlin.h>
#include "gsl_optim_ex.h"


int expb_f (const gsl_vector * x, void *data, gsl_vector * f)
{
	size_t n = ((struct data *)data)->n;
    double *y = ((struct data *)data)->y;
    double *sigma = ((struct data *) data)->sigma;
     
    double A = gsl_vector_get (x, 0);
    double lambda = gsl_vector_get (x, 1);
    double b = gsl_vector_get (x, 2);
     
    size_t i;
    
    for (i = 0; i < n; i++)
    {
		/* Model Yi = A * exp(-lambda * i) + b */
        double t = i;
        double Yi = A * exp (-lambda * t) + b;
        gsl_vector_set (f, i, (Yi - y[i])/sigma[i]);
    }
     
    return GSL_SUCCESS;
}
     
int expb_df (const gsl_vector * x, void *data, gsl_matrix * J)
{
	size_t n = ((struct data *)data)->n;
    double *sigma = ((struct data *) data)->sigma;
     
    double A = gsl_vector_get (x, 0);
    double lambda = gsl_vector_get (x, 1);
     
    size_t i;
     
    for (i = 0; i < n; i++)
    {
		/* Jacobian matrix J(i,j) = dfi / dxj, */
        /* where fi = (Yi - yi)/sigma[i],      */
        /*       Yi = A * exp(-lambda * i) + b  */
        /* and the xj are the parameters (A,lambda,b) */
        double t = i;
        double s = sigma[i];
        double e = exp(-lambda * t);
        gsl_matrix_set (J, i, 0, e/s); 
        gsl_matrix_set (J, i, 1, -t * A * e/s);
        gsl_matrix_set (J, i, 2, 1/s);
        }
       return GSL_SUCCESS;
	}
     
int expb_fdf (const gsl_vector * x, void *data, gsl_vector * f, gsl_matrix * J)
{
	expb_f (x, data, f);
	expb_df (x, data, J);
     
    return GSL_SUCCESS;
}